var currentUrl = RAMS.Environment.SharePoint.URL;
var restPath = RAMS.Environment.WebService.URL;

var requestDigest;
var subsiteUrl, subsiteTitle;
var availableStudies;
var overallProgress = 0;
var facilityID, facilityName;
var alignedUserLogins = {}; // normalized e-mail addresses and Active Directory domain\usernames

$(document).ready(function(){
    sharepointAuth();
    getSubsite();
});

function searchArchive(){
    $(".console-body").append("Acting on <a href='/sites/IRBApplications/" + subsiteUrl + "'>" + subsiteTitle + "</a> (committee) and " + facilityName + " (facility).<br/>");
    sendREST("GET", restPath.concat("unclaimed"), null, function(returnedStudies) {
	var i, studyTable, row, piDisplayName;
        availableStudies = returnedStudies;

        if (returnedStudies.length > 0) {
            $(".console-body").append("Found " + returnedStudies.length + " studies:<br>");
            $(".console-body").append("<table id=returnedStudies><thead><tr><th>Include?</th><th>ID</th><th>Title</th><th>PI login</th></tr></thead></table>");
	    studyTable = $("#returnedStudies");

	    for (i = 0; i < returnedStudies.length > 0; i++) {
		piDisplayName = returnedStudies[i].studyStatusCollection[0].createdBy.displayName;
		row = "<tr><td><input type=checkbox";
		if (alignedUserLogins[piDisplayName.toLowerCase()])
		    row += " checked";
		row += " study='" + returnedStudies[i].assignedID + "'></td>";
		row += "<td>" + returnedStudies[i].assignedID + "</td>";
		row += "<td>" + returnedStudies[i].title + "</td>";
		row += "<td>" + piDisplayName + "</td></tr>";
		studyTable.append(row);
            }
            $(".button-body").append(createButton("Import Archived Files To SharePoint", "kickoffImport()"));
	}
    });
}

function kickoffImport(){
    // console.log("pressed the button");
    formatArchive(availableStudies);
    $(".console-body").append("<hr><br>Importing " + availableStudies.length + " Studies...<br>");
    updateProgress(0);
}

function formatArchive(archivedStudies){
    var data = null;
    var mainData = null;
    var appLink = null;

	//console.log("we were passed this many studies: " + archivedStudies.length);

    while (archivedStudies.length > 0 && !$("input[type='checkbox'][study='" + archivedStudies[0].assignedID + "']").is(":checked")) {
	overallProgress++;
	updateProgress(((overallProgress)/(availableStudies.length))*100);
	archivedStudies = archivedStudies.slice(1, archivedStudies.length);
    }
	if(archivedStudies.length > 0){	
        //console.log("the study status collection is: " + JSON.stringify(archivedStudies[0].studyStatusCollection));
        titleToPrincipalId(archivedStudies[0].studyStatusCollection[0].createdBy.displayName, function(pid, user) {

            if(pid != null){
                $(".console-body").append("Importing: " + archivedStudies[0].title);

                var assignedID = archivedStudies[0].assignedID;

	            appLink = {
	                "__metadata": {"type": "SP.FieldUrlValue"},
	                "Description": archivedStudies[0].studyStatusCollection[0].createdBy.displayName + " - " + archivedStudies[0].title + ": " + assignedID,
	                "Url": currentUrl+"SiteAssets/irb-applications-general.html?" + assignedID
	            };

                mainData = {
                    "__metadata": {"type": "SP.Data.ApplicationListItem"},
                    "Study_ID": assignedID,
                    "StudyTitle": archivedStudies[0].title,
                    "Title": archivedStudies[0].title,
                    "ApplicationStatus": "Approved" 
                };
                mainData[RAMS.Environment.SharePoint.List.Application.FacilityLookupId] = facilityID;
                mainData[RAMS.Environment.SharePoint.List.Application.PrincipalInvestigatorId] = pid;
                mainData[RAMS.Environment.SharePoint.List.Application.LinkToApplication] = appLink;

                var studyMembers = {
                    "user" : user,
                    "activeMember": true,
                    "vaUser": true,
                    "teamMember": true,
                    "memberRole": "Principal Investigator",
                    "coiRequired": false,
                    "writePermissions": true,
                    "explaination": "n/a",
                    "notifications": true 
                }

                // console.log("we want to send: " + JSON.stringify(mainData)); 

                sendREST("POST", currentUrl.concat("_api/web/lists/getbytitle('Application')/items"), mainData, function(){
                    sendREST("POST", currentUrl.concat(subsiteUrl.concat("_api/web/lists/getbytitle('Committee Topics')/items")), mainData, function(){
                        var baseFolder = "Shared%20Documents/" + assignedID;
                        sendREST("POST", currentUrl.concat("_api/Web/Folders/Add('" + baseFolder + "')"), null, function() {
	                        sendREST("POST", currentUrl.concat("_api/Web/Folders/Add('" + baseFolder + "/1')"), null, function() {
		                        sendREST("POST", currentUrl.concat("_api/Web/Folders/Add('" + baseFolder + "/Study%20Team')"), null, function() {
			                        sendREST("POST", currentUrl.concat("_api/Web/Folders/Add('" + baseFolder + "/Review%20Team')"), null, function() {
				                        sendREST("POST", restPath.concat("study/"+assignedID+"/stage/Approved"), null, function(){
				                            sendREST("POST", restPath.concat("study/"+assignedID+"/members"), studyMembers, function(){
				                                overallProgress++;
				                                updateProgress(((overallProgress)/(availableStudies.length))*100);
				                                formatArchive(archivedStudies.slice(1, archivedStudies.length)); 
				                            });                           
				                        });
			                        });
		                        });
			                });
				        });                            
                    });
                });
                $(".console-body").append(" Successful...<br>");
            }else{
               $(".console-body").append("Cannot import " + archivedStudies[0].title + "! Incorrectly formatted data [No Principal Investigator] - Skipping Record...<br>");
                //have to make call again in case there are still studies that can be sent
                overallProgress++;
                updateProgress(((overallProgress)/(availableStudies.length))*100);
                formatArchive(archivedStudies.slice(1, archivedStudies.length));
            }
        });
    }else{
       $(".console-body").append("Import Complete...<br>");
        overallProgress++;
        updateProgress(((overallProgress)/(availableStudies.length))*100);
    }

	
};

function titleToPrincipalId(userTitle, resultFunction){
	var prefix = "i:0#.w|";
	sendREST("POST", currentUrl + "_api/Web/EnsureUser", { "logonName": prefix + userTitle }, function(ensuredUser) {
		if (ensuredUser) {
			sendREST("POST", restPath + "user?loginName=" + ensuredUser.d.LoginName.replace(prefix, ""), null, function(webAppUser) {
				resultFunction(ensuredUser.d.Id, webAppUser);
			});
		} else {
			resultFunction(null, null);
		}
	});
}

function sharepointAuth(){
	sendREST("POST", currentUrl.concat("_api/contextinfo"), null, function(returnedContext){
		requestDigest = returnedContext.d.GetContextWebInformation.FormDigestValue;
	});
};

var groupId;
function getSubsite(){
    sendREST("GET", currentUrl.concat("_api/Web/CurrentUser/Groups"), null, function(returnedGroups){
        var currentGroups = returnedGroups.d.results, i;
        //console.log("you are in the groups: " + JSON.stringify(currentGroups));

        for (i = 0; !facilityID && i < currentGroups.length; i++) {
            if(currentGroups[i].Title.indexOf("Users") > -1){
                groupId = currentGroups[i].Id;

                sendREST("GET", currentUrl.concat("_api/Web/lists/getbytitle('Committee List')/items"), null, function(returnedCommittees){
                    var availableCommittees = returnedCommittees.d.results, i;

                    //console.log("the current committees are : " + JSON.stringify(availableCommittees));
                    for (i = 0; !facilityID && i < availableCommittees.length; i++) {
                        if (groupId == availableCommittees[i].UsersId) {
			    subsiteTitle = availableCommittees[i].Title;
                            subsiteUrl = availableCommittees[i].CommitteeSub_x002d_Site.Url;
                            subsiteUrl = subsiteUrl.replace(currentUrl, "");
                            subsiteUrl = subsiteUrl + "/";

                            getFacility(availableCommittees[i].Id);
                        }
                    }
                });
            }
        }
    });
};

function getFacility(committeeID){
    sendREST("GET", currentUrl.concat("_api/web/lists/getbytitle('Facility')/items"), null, function(returnedFacilities){
        var availableFacilities = returnedFacilities.d.results, i;
        for (i = 0; !facilityID && i < availableFacilities.length; i++) {
            if (availableFacilities[i].CommitteeLookupId == committeeID) {
                facilityID = availableFacilities[i].Id;
		facilityName = availableFacilities[i].FacilityName;
		sendREST("GET", currentUrl + "_api/Web/SiteGroups/GetById(" + groupId + ")/Users", null, function(users) {
		    var i;
		    users = users.d.results;
		    for (i = 0; i < users.length; i++)
			alignedUserLogins[users[i].Email.toLowerCase()] = alignedUserLogins[users[i].LoginName.replace("i:0#.w|", "")] = true;
	            searchArchive();
		});
            }
        }
    });
};

function createButton(buttonTitle, onclickFunc){
    var button, text; 

    button = document.createElement("button");
    button.setAttribute("type", "button");
    button.setAttribute("title", buttonTitle);
    button.setAttribute("onclick", onclickFunc);

    text = document.createTextNode(buttonTitle);

    button.appendChild(text);

    return button;
};

// function archiveButton(){
//     $("body").append(createButton("Import Archived Files To SharePoint", "importKickoff()"));
// };

function updateProgress(currentProgress){
    var container, progressBar, progressSpan, text;

    $(".progress").remove();

    // console.log("updating progress. currentProgress: " + currentProgress + "/" + availableStudies.length);

    container = document.createElement("div");
    container.setAttribute("class", "progress");

    progressBar = document.createElement("div");
    progressBar.setAttribute("class", "progress-bar progress-bar-striped active");
    progressBar.setAttribute("role", "progressbar");
    progressBar.setAttribute("aria-valuenow", currentProgress);
    progressBar.setAttribute("aria-valuemin", "0");
    progressBar.setAttribute("aria-valuemax", availableStudies.length);
    progressBar.setAttribute("style", "width: "+currentProgress+"%;");

    progressSpan = document.createElement("span");
    progressSpan.setAttribute("class", "sr-only");

    text = document.createTextNode(currentProgress+"% Complete");
    progressSpan.appendChild(text);
    progressBar.appendChild(progressSpan);
    container.appendChild(progressBar);
    $('.progress-body').append(container);
}

//connects to a rest endpoint and either sends or
//recieves data
function sendREST(httpMethod, endpoint, data, callback, spHttpMethod){
    var xmlResponse;
    
    //accounting for internet explorer's needs
    if(window.XMLHttpRequest){
    xmlResponse = new XMLHttpRequest();
    }else{
    xmlResponse = new ActiveXObject("Microsoft.XMLHTTP");
    }

    try{
        //setting up general headers
        xmlResponse.open(httpMethod, endpoint);
        xmlResponse.withCredentials = true;
        // xmlResponse.setRequestHeader("Content-Type", "application/json");
        xmlResponse.setRequestHeader("Accept", "application/json; odata=verbose");
        xmlResponse.setRequestHeader("X-RequestDigest", requestDigest);

		if(httpMethod == "POST"){
            try{
                xmlResponse.setRequestHeader("Content-Type", "application/json; odata=verbose");
                
                if(spHttpMethod == "Merge"){
                    xmlResponse.setRequestHeader("X-HTTP-Method", "MERGE");
                    xmlResponse.setRequestHeader("If-Match", "*");
                }else if(spHttpMethod == "Delete"){
                    xmlResponse.setRequestHeader("X-HTTP-Method", "DELETE");
                }

                //sending data and alerting the user
                if(data){
                    xmlResponse.send(JSON.stringify(data));
                }else{
                    xmlResponse.send(null);
                }

                if(callback){
                    xmlResponse.onreadystatechange = function(){
                        if(xmlResponse.readyState === 4){
                            if((xmlResponse.status === 200) && (xmlResponse.responseText)){
                                //response has content
                                xmlResponse = JSON.parse(xmlResponse.responseText);
                                callback(xmlResponse);
                            }else{
                                callback();
                            }
                        }
                    };
                }
            }catch(e){
                alert(e.toString());
            }
        }else if(httpMethod == "GET"){
            //retrieving the needed data
            xmlResponse.onreadystatechange = function(){
                if(xmlResponse.readyState === 4){
                    if(xmlResponse.status === 200){ 
                        try{
                             xmlResponse = JSON.parse(xmlResponse.responseText);
                             if(callback)
                                callback(xmlResponse);
                        }catch(e){
                            alert(e.toString());
                        }   
                    }else{
                        if((xmlResponse.status !== 204) && (xmlResponse.status !== 1223))
                         alert(xmlResponse.status);
                    }
                }
            };
            xmlResponse.send(null);
            //passing the needed data to the function that is requesting it
            return xmlResponse;
        }

    }catch(e){
       alert(e.toString());
    }
};
