//waiting until sharepoint has loaded the required files.
$(document).ready(function(){
    process();
});

//Linking in the web service
var restPath = RAMS.Environment.WebService.URL;

//Linking in the sharepoint site
var siteUrl = RAMS.Environment.SharePoint.URL;

//template for username that is retrieved from sharepoint 
var retrievedUser;
var user;
//globals that have to be shared between functions
var facility, id;
var studyID = 1;

//Initial function called by HTML. kicks off the process
//to grab the username from sharepoint
function process(){
    //loading connection to sharepoint
    var clientContext = new SP.ClientContext(siteUrl);

    this.website = clientContext.get_web();
    this.currentUser_SP = website.get_currentUser();
    clientContext.load(currentUser_SP);
    //trying to grab username information from sharepoint
    clientContext.executeQueryAsync(Function.createDelegate(this, this.userQuerySucceeded), Function.createDelegate(this, this.userQueryFailed));
};

//When we know that we have successfully retrieved a username from
//shareponit, then we can set up the baseline information that is needed.
function userQuerySucceeded(sender, args) {
    var username_SP, split;
user = currentUser_SP;
    //parsing out the needed username from sharepoint
    username_SP = currentUser_SP.get_loginName();
    split = username_SP.split("|");
    retrievedUser = split[1];

    id = 1;
    //puts together the dashboard table
    sendREST("POST", restPath.concat("user"), null, function(user){   
        createTable();

        //removes HTML traces that allow user to leave dashboard page
         $('.start').attr("href", "irb-applications-general.html?" + id);
        //$('.start').attr("href", "irb-applications-general.html?" + id);
        $('.start').html("Start IRB Application");
    });
};

//notifying the user that their sharepoint username couldn't be retrieved
function userQueryFailed(sender, args) {
    alert('request failed ' + args.get_message() + '\n' + args.get_stackTrace());
};

//puts together the dashboard table. This will list out any studies that
//the user is associated with and provide information on those studies.
function createTable(){
    var recentStage, row;
    var noStudies = true;
    
    //grabs all the studies that have been created.
    sendREST("GET", restPath.concat("manage/study"), null, function (studies){   
        //filtering through the studies and putting the necessary ones in the dashboard
        for(var i=0; i<studies.length; i++){

            //recentStage relates to whether or not the study is in draft or review
        	recentStage = studies[i].studyStatusCollection.length-1;
        		    
            // if(recentStage < 0){
            //     alert("bad study: " + studies[i].assignedID);
            // }

            /*temp adding users to study member collection*/
            var foundUser = false;
            ////console.log("collection length: " + studies[i].studyMembersCollection.length);
            for(var j=0; j<studies[i].studyMembersCollection.length; j++){
                // //console.log("checking " + studies[i].studyMembersCollection[j].user.loginName + " against " + retrievedUser);
                if(studies[i].studyMembersCollection[j].user.loginName == retrievedUser){
                    // //console.log("found user");
                    foundUser = true;
                }
            }

        	row = document.createElement("tr");
            if((foundUser) || ((recentStage >= 0)&&(studies[i].studyStatusCollection[recentStage].createdBy.loginName == retrievedUser))){

        	    var cell = document.createElement("td");
        	    
                //allows the user to return to a previous study by clicking on this study ID link in the dashboard
        	    var anchor = document.createElement("a");
        	    anchor.setAttribute("href", "irb-applications-general.html?" + studies[i].assignedID);
        	    anchor.setAttribute("title", studies[i].title);
        	    
                //study ID
        	    var text = document.createTextNode(studies[i].assignedID);
        	    anchor.appendChild(text);
        	    cell.appendChild(anchor);
        	    row.appendChild(cell);
        	    
        	    //study title
        	    row.appendChild(createCell(studies[i].title));
        	    
        	    //Type 
        	    row.appendChild(createCell("Type TBD"));
        	    
        	    //PI Name
        	    row.appendChild(createCell(studies[i].studyStatusCollection[recentStage].createdBy.displayName));
        	    
        	    //Status (draft or review)
        	    row.appendChild(createCell(studies[i].studyStatusCollection[recentStage].studyState.name));
        	    
        	    //Expiration Date
        	    row.appendChild(createCell("Expiration TBD"));
        	    
        	    //IRB Agenda Date
        	    row.appendChild(createCell("IRB DATE TBD"));
        	
        	    $('tbody').append(row);
        	    noStudies = false;
        	}	
        }

        if(noStudies){
        	//no studies linked to user
            row = document.createElement("tr");
        	var cell = document.createElement("td")
            var text = document.createTextNode("You have no applications");
            cell.appendChild(text);
            cell.setAttribute("colspan", 7);
        	cell.setAttribute("style", "text-align:center");
        	row.appendChild(cell);	    
            
        	$('tbody').append(row);
        }
    });
};

//creates a cell in a table. takes in what should be shown
//in the cell
function createCell(displayText){
    
    var cell = document.createElement("td");
    
    var text = document.createTextNode(displayText);
    cell.appendChild(text);
    
    return cell;
};

//connects to a rest endpoint and either sends or
//recieves data
function sendREST(httpMethod, endpoint, data, callback){
    var xmlResponse;

    //accounting for internet explorer's needs
    if(window.XMLHttpRequest){
        xmlResponse = new XMLHttpRequest();
    }else{
        xmlResponse = new ActiveXObject("Microsoft.XMLHTTP");
    }

    try{
        //setting up general headers
        xmlResponse.open(httpMethod, endpoint);
        xmlResponse.setRequestHeader("Content-Type", "application/json");
        xmlResponse.setRequestHeader("Accept", "application/json");
		xmlResponse.withCredentials = true;

        //retrieving the needed data
        xmlResponse.onreadystatechange = function(){
            if(xmlResponse.readyState === 4){
                if(xmlResponse.status === 200){
                    try{
                        if(xmlResponse.responseText)
                            xmlResponse = JSON.parse(xmlResponse.responseText);
        			    if (callback !== undefined){
        				    callback(xmlResponse);
                        }
                    }catch(e){
                        alert(e.toString());
                    }

                }else{
                    alert(xmlResponse.statusText);
                }
            }
        };
        xmlResponse.send(null);
        //passing the needed data to the function that is requesting it
        return xmlResponse;
    }catch(e){
        alert(e.toString());
    }
};

